package com.vseravno.solna;

import com.vseravno.solna.adapter.*;
import com.vseravno.solna.handler.CompiledHandler;
import com.vseravno.solna.handler.HandlerCompiler;
import com.vseravno.solna.parser.XmlParser;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

@ThreadSafe
public class SolnaParser {
    private static final List<Adapter> defaultAdapters = new ArrayList<Adapter>();

    static {
        defaultAdapters.addAll(Arrays.asList(
                new W3cAdapter(),
                new Jdom1Adapter(),
                new Jdom2Adapter(),
                new Dom4jAdapter(),
                new XomAdapter(),
                new StringAdapter()
        ));
    }

    private static final HandlerCompiler handlerCompiler = new HandlerCompiler();

    private final List<CompiledHandler> handlers = new CopyOnWriteArrayList<CompiledHandler>();

    public void parse(@Nonnull InputStream input) throws SolnaException {
        try {
            new XmlParser(input, handlers).parse();

        } catch (SolnaException e) {
            throw e;
        } catch (Exception e) {
            throw new SolnaException("Couldn't parse input stream", e);
        }
    }

    public void addHandler(@Nonnull String elementPath, @Nonnull SolnaHandler<?> handler) throws SolnaException {
        try {
            CompiledHandler compiledHandler = handlerCompiler.compile(elementPath, handler, defaultAdapters);

            handlers.add(compiledHandler);

        } catch (SolnaException e) {
            throw e;
        } catch (Exception e) {
            throw new SolnaException("Couldn't add element handler", e);
        }
    }
}
